// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

use crate::{ffi, VideoAggregatorConvertPad, VideoAggregatorPad};

glib::wrapper! {
    #[doc(alias = "GstVideoAggregatorParallelConvertPad")]
    pub struct VideoAggregatorParallelConvertPad(Object<ffi::GstVideoAggregatorParallelConvertPad, ffi::GstVideoAggregatorParallelConvertPadClass>) @extends VideoAggregatorConvertPad, VideoAggregatorPad, gst_base::AggregatorPad, gst::Pad, gst::Object;

    match fn {
        type_ => || ffi::gst_video_aggregator_parallel_convert_pad_get_type(),
    }
}

impl VideoAggregatorParallelConvertPad {
    pub const NONE: Option<&'static VideoAggregatorParallelConvertPad> = None;
}

unsafe impl Send for VideoAggregatorParallelConvertPad {}
unsafe impl Sync for VideoAggregatorParallelConvertPad {}
