// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

#[allow(unused_imports)]
use crate::auto::*;

#[doc(alias = "GstClockTimeDiff")]
pub type ClockTimeDiff = i64;
