// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "GstAudioVisualizer", sizeof(GstAudioVisualizer), alignof(GstAudioVisualizer));
    printf("%s;%zu;%zu\n", "GstAudioVisualizerClass", sizeof(GstAudioVisualizerClass), alignof(GstAudioVisualizerClass));
    printf("%s;%zu;%zu\n", "GstAudioVisualizerShader", sizeof(GstAudioVisualizerShader), alignof(GstAudioVisualizerShader));
    printf("%s;%zu;%zu\n", "GstDiscoverer", sizeof(GstDiscoverer), alignof(GstDiscoverer));
    printf("%s;%zu;%zu\n", "GstDiscovererAudioInfoClass", sizeof(GstDiscovererAudioInfoClass), alignof(GstDiscovererAudioInfoClass));
    printf("%s;%zu;%zu\n", "GstDiscovererClass", sizeof(GstDiscovererClass), alignof(GstDiscovererClass));
    printf("%s;%zu;%zu\n", "GstDiscovererContainerInfoClass", sizeof(GstDiscovererContainerInfoClass), alignof(GstDiscovererContainerInfoClass));
    printf("%s;%zu;%zu\n", "GstDiscovererInfoClass", sizeof(GstDiscovererInfoClass), alignof(GstDiscovererInfoClass));
    printf("%s;%zu;%zu\n", "GstDiscovererResult", sizeof(GstDiscovererResult), alignof(GstDiscovererResult));
    printf("%s;%zu;%zu\n", "GstDiscovererSerializeFlags", sizeof(GstDiscovererSerializeFlags), alignof(GstDiscovererSerializeFlags));
    printf("%s;%zu;%zu\n", "GstDiscovererStreamInfoClass", sizeof(GstDiscovererStreamInfoClass), alignof(GstDiscovererStreamInfoClass));
    printf("%s;%zu;%zu\n", "GstDiscovererSubtitleInfoClass", sizeof(GstDiscovererSubtitleInfoClass), alignof(GstDiscovererSubtitleInfoClass));
    printf("%s;%zu;%zu\n", "GstDiscovererVideoInfoClass", sizeof(GstDiscovererVideoInfoClass), alignof(GstDiscovererVideoInfoClass));
    printf("%s;%zu;%zu\n", "GstEncodingTargetClass", sizeof(GstEncodingTargetClass), alignof(GstEncodingTargetClass));
    printf("%s;%zu;%zu\n", "GstInstallPluginsReturn", sizeof(GstInstallPluginsReturn), alignof(GstInstallPluginsReturn));
    printf("%s;%zu;%zu\n", "GstPbUtilsCapsDescriptionFlags", sizeof(GstPbUtilsCapsDescriptionFlags), alignof(GstPbUtilsCapsDescriptionFlags));
    return 0;
}
