// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

use crate::{ffi, EncodingProfile};

glib::wrapper! {
    #[doc(alias = "GstEncodingAudioProfile")]
    pub struct EncodingAudioProfile(Object<ffi::GstEncodingAudioProfile, ffi::GstEncodingAudioProfileClass>) @extends EncodingProfile;

    match fn {
        type_ => || ffi::gst_encoding_audio_profile_get_type(),
    }
}

impl EncodingAudioProfile {}

unsafe impl Send for EncodingAudioProfile {}
unsafe impl Sync for EncodingAudioProfile {}
