// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "GstAggregator", sizeof(GstAggregator), alignof(GstAggregator));
    printf("%s;%zu;%zu\n", "GstAggregatorClass", sizeof(GstAggregatorClass), alignof(GstAggregatorClass));
    printf("%s;%zu;%zu\n", "GstAggregatorPad", sizeof(GstAggregatorPad), alignof(GstAggregatorPad));
    printf("%s;%zu;%zu\n", "GstAggregatorPadClass", sizeof(GstAggregatorPadClass), alignof(GstAggregatorPadClass));
    printf("%s;%zu;%zu\n", "GstAggregatorStartTimeSelection", sizeof(GstAggregatorStartTimeSelection), alignof(GstAggregatorStartTimeSelection));
    printf("%s;%zu;%zu\n", "GstBaseParse", sizeof(GstBaseParse), alignof(GstBaseParse));
    printf("%s;%zu;%zu\n", "GstBaseParseClass", sizeof(GstBaseParseClass), alignof(GstBaseParseClass));
    printf("%s;%zu;%zu\n", "GstBaseParseFrame", sizeof(GstBaseParseFrame), alignof(GstBaseParseFrame));
    printf("%s;%zu;%zu\n", "GstBaseParseFrameFlags", sizeof(GstBaseParseFrameFlags), alignof(GstBaseParseFrameFlags));
    printf("%s;%zu;%zu\n", "GstBaseSink", sizeof(GstBaseSink), alignof(GstBaseSink));
    printf("%s;%zu;%zu\n", "GstBaseSinkClass", sizeof(GstBaseSinkClass), alignof(GstBaseSinkClass));
    printf("%s;%zu;%zu\n", "GstBaseSrc", sizeof(GstBaseSrc), alignof(GstBaseSrc));
    printf("%s;%zu;%zu\n", "GstBaseSrcClass", sizeof(GstBaseSrcClass), alignof(GstBaseSrcClass));
    printf("%s;%zu;%zu\n", "GstBaseSrcFlags", sizeof(GstBaseSrcFlags), alignof(GstBaseSrcFlags));
    printf("%s;%zu;%zu\n", "GstBaseTransform", sizeof(GstBaseTransform), alignof(GstBaseTransform));
    printf("%s;%zu;%zu\n", "GstBaseTransformClass", sizeof(GstBaseTransformClass), alignof(GstBaseTransformClass));
    printf("%s;%zu;%zu\n", "GstBitReader", sizeof(GstBitReader), alignof(GstBitReader));
    printf("%s;%zu;%zu\n", "GstBitWriter", sizeof(GstBitWriter), alignof(GstBitWriter));
    printf("%s;%zu;%zu\n", "GstByteReader", sizeof(GstByteReader), alignof(GstByteReader));
    printf("%s;%zu;%zu\n", "GstByteWriter", sizeof(GstByteWriter), alignof(GstByteWriter));
    printf("%s;%zu;%zu\n", "GstCollectData", sizeof(GstCollectData), alignof(GstCollectData));
    printf("%s;%zu;%zu\n", "GstCollectPads", sizeof(GstCollectPads), alignof(GstCollectPads));
    printf("%s;%zu;%zu\n", "GstCollectPadsClass", sizeof(GstCollectPadsClass), alignof(GstCollectPadsClass));
    printf("%s;%zu;%zu\n", "GstCollectPadsStateFlags", sizeof(GstCollectPadsStateFlags), alignof(GstCollectPadsStateFlags));
    printf("%s;%zu;%zu\n", "GstDataQueue", sizeof(GstDataQueue), alignof(GstDataQueue));
    printf("%s;%zu;%zu\n", "GstDataQueueClass", sizeof(GstDataQueueClass), alignof(GstDataQueueClass));
    printf("%s;%zu;%zu\n", "GstDataQueueItem", sizeof(GstDataQueueItem), alignof(GstDataQueueItem));
    printf("%s;%zu;%zu\n", "GstDataQueueSize", sizeof(GstDataQueueSize), alignof(GstDataQueueSize));
    printf("%s;%zu;%zu\n", "GstPushSrc", sizeof(GstPushSrc), alignof(GstPushSrc));
    printf("%s;%zu;%zu\n", "GstPushSrcClass", sizeof(GstPushSrcClass), alignof(GstPushSrcClass));
    return 0;
}
