// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::{ffi, Accessible, Buildable, ConstraintTarget, Orientable, Scrollable, Widget};

glib::wrapper! {
    #[doc(alias = "GtkListBase")]
    pub struct ListBase(Object<ffi::GtkListBase, ffi::GtkListBaseClass>) @extends Widget, @implements Accessible, Buildable, ConstraintTarget, Orientable, Scrollable;

    match fn {
        type_ => || ffi::gtk_list_base_get_type(),
    }
}

impl ListBase {
    pub const NONE: Option<&'static ListBase> = None;
}
