// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

use crate::{ffi, DiscovererStreamInfo};
use glib::translate::*;

glib::wrapper! {
    #[doc(alias = "GstDiscovererAudioInfo")]
    pub struct DiscovererAudioInfo(Object<ffi::GstDiscovererAudioInfo>) @extends DiscovererStreamInfo;

    match fn {
        type_ => || ffi::gst_discoverer_audio_info_get_type(),
    }
}

impl DiscovererAudioInfo {
    #[doc(alias = "gst_discoverer_audio_info_get_bitrate")]
    #[doc(alias = "get_bitrate")]
    pub fn bitrate(&self) -> u32 {
        unsafe { ffi::gst_discoverer_audio_info_get_bitrate(self.to_glib_none().0) }
    }

    #[doc(alias = "gst_discoverer_audio_info_get_channel_mask")]
    #[doc(alias = "get_channel_mask")]
    pub fn channel_mask(&self) -> u64 {
        unsafe { ffi::gst_discoverer_audio_info_get_channel_mask(self.to_glib_none().0) }
    }

    #[doc(alias = "gst_discoverer_audio_info_get_channels")]
    #[doc(alias = "get_channels")]
    pub fn channels(&self) -> u32 {
        unsafe { ffi::gst_discoverer_audio_info_get_channels(self.to_glib_none().0) }
    }

    #[doc(alias = "gst_discoverer_audio_info_get_depth")]
    #[doc(alias = "get_depth")]
    pub fn depth(&self) -> u32 {
        unsafe { ffi::gst_discoverer_audio_info_get_depth(self.to_glib_none().0) }
    }

    #[doc(alias = "gst_discoverer_audio_info_get_language")]
    #[doc(alias = "get_language")]
    pub fn language(&self) -> Option<glib::GString> {
        unsafe {
            from_glib_none(ffi::gst_discoverer_audio_info_get_language(
                self.to_glib_none().0,
            ))
        }
    }

    #[doc(alias = "gst_discoverer_audio_info_get_max_bitrate")]
    #[doc(alias = "get_max_bitrate")]
    pub fn max_bitrate(&self) -> u32 {
        unsafe { ffi::gst_discoverer_audio_info_get_max_bitrate(self.to_glib_none().0) }
    }

    #[doc(alias = "gst_discoverer_audio_info_get_sample_rate")]
    #[doc(alias = "get_sample_rate")]
    pub fn sample_rate(&self) -> u32 {
        unsafe { ffi::gst_discoverer_audio_info_get_sample_rate(self.to_glib_none().0) }
    }
}

unsafe impl Send for DiscovererAudioInfo {}
unsafe impl Sync for DiscovererAudioInfo {}
