//! This file is generated code. Please edit the generator
//! in dev-tools/gen-target-info if you need to make changes.

pub const RISCV_ARCH_MAPPING: &[(&str, &str)] = &[
    ("riscv32gc", "riscv32"),
    ("riscv32i", "riscv32"),
    ("riscv32im", "riscv32"),
    ("riscv32ima", "riscv32"),
    ("riscv32imac", "riscv32"),
    ("riscv32imafc", "riscv32"),
    ("riscv32imc", "riscv32"),
    ("riscv64gc", "riscv64"),
    ("riscv64imac", "riscv64"),
];
pub const WINDOWS_TRIPLE_MAPPING: &[(&str, &str)] = &[
    ("aarch64-pc-windows-gnullvm", "aarch64-pc-windows-gnu"),
    ("aarch64-uwp-windows-msvc", "aarch64-pc-windows-msvc"),
    ("i686-pc-windows-gnullvm", "i686-pc-windows-gnu"),
    ("i686-uwp-windows-gnu", "i686-pc-windows-gnu"),
    ("i686-uwp-windows-msvc", "i686-pc-windows-msvc"),
    ("i686-win7-windows-msvc", "i686-pc-windows-msvc"),
    ("thumbv7a-uwp-windows-msvc", "thumbv7a-pc-windows-msvc"),
    ("x86_64-pc-windows-gnullvm", "x86_64-pc-windows-gnu"),
    ("x86_64-uwp-windows-gnu", "x86_64-pc-windows-gnu"),
    ("x86_64-uwp-windows-msvc", "x86_64-pc-windows-msvc"),
];
