// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::{ffi, Pixbuf, PixbufAnimation};
use glib::{prelude::*, translate::*};

glib::wrapper! {
    #[doc(alias = "GdkPixbufNonAnim")]
    pub struct PixbufNonAnim(Object<ffi::GdkPixbufNonAnim>) @extends PixbufAnimation;

    match fn {
        type_ => || ffi::gdk_pixbuf_non_anim_get_type(),
    }
}

impl PixbufNonAnim {
    #[doc(alias = "gdk_pixbuf_non_anim_new")]
    pub fn new(pixbuf: &Pixbuf) -> PixbufNonAnim {
        unsafe {
            PixbufAnimation::from_glib_full(ffi::gdk_pixbuf_non_anim_new(pixbuf.to_glib_none().0))
                .unsafe_cast()
        }
    }
}
